from flask import current_app
from sqlalchemy import text


class Dashboard:
    @staticmethod
    def university_main_stat(lname: str) -> dict:
        sql = """
        WITH pps AS(
        SELECT p.mid
        FROM people p
        LEFT JOIN xp_personal_file xpf
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        ),

        pps_gp AS(
            SELECT p.mid
            FROM people p
            LEFT JOIN xp_personal_file xpf
            ON p.mid = xpf.mid
            WHERE xpf.category_stat = 'Преподавательский состав'
            AND (xpf.perstype = 'Гражданский персонал' OR xpf.perstype IS NULL)
        ),
        pps_v AS (
            SELECT p.mid
            FROM people p
            LEFT JOIN xp_personal_file xpf
            ON p.mid = xpf.mid
            WHERE xpf.category_stat = 'Преподавательский состав'
            AND xpf.perstype = 'Военнослужащие'
        ),

        pps_doctor AS (
            SELECT
                xpf.mid
            FROM xp_personal_file xpf

            LEFT JOIN academicdegree ad
            ON xpf."AcademicDegree" = ad.agid
            WHERE idokin = 618
            AND xpf.category_stat = 'Преподавательский состав'
        ),

        pps_kandidat AS (
            SELECT
                xpf.mid
            FROM xp_personal_file xpf

            LEFT JOIN academicdegree ad
            ON xpf."AcademicDegree" = ad.agid
            WHERE idokin = 619
            AND xpf.category_stat = 'Преподавательский состав'
        ),
        pps_professor AS (
            SELECT
                xpf.mid
            FROM xp_personal_file xpf
            LEFT JOIN academictitle act
            ON xpf."AcademicTitle" = act.atid
            WHERE act.name = 'Профессор'
            AND xpf.category_stat = 'Преподавательский состав'
        ),

        pps_docent AS (
            SELECT
                xpf.mid
            FROM xp_personal_file xpf
            LEFT JOIN academictitle act
            ON xpf."AcademicTitle" = act.atid
            WHERE act.name = 'Доцент'
            AND xpf.category_stat = 'Преподавательский состав'
        ),

        pps_doctorant AS (
            SELECT xpf.category
            FROM xp_personal_file xpf
            LEFT JOIN prep_struc_category psc
            ON xpf.category = psc.cid
            WHERE psc.name = 'Докторант'
            AND xpf.category_stat = 'Преподавательский состав'
        ),


        pps_aduct AS (
            SELECT xpf.category
            FROM xp_personal_file xpf
            LEFT JOIN prep_struc_category psc
            ON xpf.category = psc.cid
            WHERE psc.name = 'Адъюнкт'
            AND xpf.category_stat = 'Преподавательский состав'
        )


        SELECT
            ROUND((SELECT COUNT(*) FROM pps_gp) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_civil,
            ROUND((SELECT COUNT(*) FROM pps_v) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_voin,
            ROUND((SELECT COUNT(*) FROM pps_doctor) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_doctor,
            ROUND((SELECT COUNT(*) FROM pps_kandidat) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_kandidat,
            ROUND((SELECT COUNT(*) FROM pps_docent) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_docent,
            ROUND((SELECT COUNT(*) FROM pps_doctorant) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_doctorant,
            ROUND((SELECT COUNT(*) FROM pps_aduct) * 100.0 / (SELECT COUNT(*) FROM pps))::int perc_aduct
        """
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(sql)
        pps_perc = query.first()
        if not pps_perc:
            return False
        pps_perc = dict(pps_perc)
        return pps_perc

    @staticmethod
    def get_university_age_stat(lname: str) -> dict:
        sql = """
        WITH up_tp_35_pps AS (
            SELECT COUNT(*) cnt
        FROM people p
        LEFT JOIN xp_personal_file xpf
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        AND p.birthdate > current_date - interval '35 years'
        ),

        bw_36_to_46 AS (
        SELECT COUNT(*) cnt
        FROM people p
        LEFT JOIN xp_personal_file xpf
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        AND p.birthdate BETWEEN current_date - interval '46 years' AND current_date - interval '35 years'
        ),

        bw_46_to_55 AS (
        SELECT COUNT(*) cnt
        FROM people p
        LEFT JOIN xp_personal_file xpf
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        AND p.birthdate BETWEEN current_date - interval '56 years' AND current_date - interval '46 years'
        ),

        bw_56_to_65 AS (
        SELECT COUNT(*) cnt
        FROM people p
        LEFT JOIN xp_personal_file xpf
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        AND p.birthdate BETWEEN current_date - interval '66 years' AND current_date - interval '56 years'
        ),

        over_65 AS (
        SELECT COUNT(*) cnt
        FROM people p
        LEFT JOIN xp_personal_file xpf
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        AND p.birthdate <= current_date - interval '66 years'
        )

        SELECT
        ROUND((SELECT cnt FROM up_tp_35_pps) * 100.0 / COUNT(*))::int up_to_35,
        ROUND((SELECT cnt FROM bw_36_to_46) * 100.0 / COUNT(*))::int bw_36_to_46,
        ROUND((SELECT cnt FROM bw_46_to_55) * 100.0 / COUNT(*))::int bw_46_to_55,
        ROUND((SELECT cnt FROM bw_56_to_65) * 100.0 / COUNT(*))::int bw_56_to_65,
        ROUND((SELECT cnt FROM over_65)     * 100.0 / COUNT(*))::int over_65
        FROM xp_personal_file xpf
        INNER JOIN people p
        ON p.mid = xpf.mid
        WHERE xpf.category_stat = 'Преподавательский состав'
        AND p.birthdate IS NOT NULL
        """
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(sql)
        pps_perc = query.first()
        if not pps_perc:
            return False
        pps_perc = dict(pps_perc)
        return pps_perc

    @staticmethod
    def get_university_mark_stat(lname) -> list:
        sql = """
        WITH aug_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-08-01' AND '2021-08-31'
        ),

        sep_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-09-01' AND '2021-09-30'
        ),

        oct_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-10-01' AND '2021-10-31'
        ),

        nov_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-11-01' AND '2021-11-30'
        ),

        decb_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-12-01' AND '2021-12-31'
        ),

        jan_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-01-01' AND '2022-01-31'
        ),

        feb_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-02-01' AND '2022-02-28'
        ),

        mar_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-03-01' AND '2022-03-31'
        ),

        apr_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-04-01' AND '2022-04-30'
        ),

        may_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-05-01' AND '2022-05-31'
        ),

        jun_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-06-01' AND '2022-06-30'
        ),

        jul_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-07-01' AND '2022-07-31'
        )


        SELECT
            aug_grade.grade aug,
            sep_grade.grade sep,
            oct_grade.grade oct,
            nov_grade.grade nov,
            decb_grade.grade decb,
            jan_grade.grade jan,
            feb_grade.grade feb,
            mar_grade.grade mar,
            apr_grade.grade apr,
            may_grade.grade may,
            jun_grade.grade jun,
            jul_grade.grade jul

            FROM
            aug_grade,
            sep_grade,
            oct_grade,
            nov_grade,
            decb_grade,
            jan_grade,
            feb_grade,
            mar_grade,
            apr_grade,
            may_grade,
            jun_grade,
            jul_grade
        """

        conn = current_app.ms.db(lname).connect()
        query = conn.execute(sql)
        mark_stat = query.first()
        if not mark_stat:
            return False
        return dict(mark_stat)

    @staticmethod
    def get_faculty_mark_stat(lname, faculty_id: int) -> list:
        sql = """
        WITH faculty_mids AS (
            SELECT
            gu.mid

            FROM groupuser gu
            LEFT JOIN groupname gn
            ON gu.gid = gn.gid

            WHERE gn.idfaculty = {faculty_id}
        ),

        aug_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE g.grade_date BETWEEN '2021-08-01' AND '2021-08-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        sep_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-09-01' AND '2021-09-30'
                AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        oct_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-10-01' AND '2021-10-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        nov_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-11-01' AND '2021-11-30'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        decb_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2021-12-01' AND '2021-12-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        jan_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-01-01' AND '2022-01-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        feb_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-02-01' AND '2022-02-28'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        mar_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-03-01' AND '2022-03-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        apr_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-04-01' AND '2022-04-30'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        may_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-05-01' AND '2022-05-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        jun_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-06-01' AND '2022-06-30'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        ),

        jul_grade AS (
            SELECT
            ROUND(CAST(SUM(xp_sum_grade(g.grade))/NULLIF(SUM(xp_grades_count(g.grade)),0) AS NUMERIC),2)::float grade
            FROM nnz_sh_grades g
            WHERE grade_date BETWEEN '2022-07-01' AND '2022-07-31'
            AND g.student_mid IN (SELECT mid FROM faculty_mids)
        )


        SELECT
            aug_grade.grade aug,
            sep_grade.grade sep,
            oct_grade.grade oct,
            nov_grade.grade nov,
            decb_grade.grade decb,
            jan_grade.grade jan,
            feb_grade.grade feb,
            mar_grade.grade mar,
            apr_grade.grade apr,
            may_grade.grade may,
            jun_grade.grade jun,
            jul_grade.grade jul

        FROM
            aug_grade,
            sep_grade,
            oct_grade,
            nov_grade,
            decb_grade,
            jan_grade,
            feb_grade,
            mar_grade,
            apr_grade,
            may_grade,
            jun_grade,
            jul_grade
        """.format(
            faculty_id=faculty_id
        )
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(sql)
        mark_stat = query.first()
        if not mark_stat:
            return False
        return dict(mark_stat)
