# LMSAPI/API/Faculty.py

from typing import Optional
from sqlalchemy import create_engine
from sqlalchemy.sql import text
from LMSAPI.api.Models.User import User
from flask import current_app, g
import pickle


class Group:
    def __init__(self, cname):
        self.gid = 0
        self.name = ""
        self.lname = cname

    def serialize(self):
        return {"gid": self.gid, "name": self.name}

    def get_group_list_from_redis(self, cathedraid):
        cachedGlist = current_app.ms.redis(self.lname).get("SYSGLIST" + str(cathedraid))
        if cachedGlist is not None:
            glist = pickle.loads(cachedGlist)
            return glist
        return

    def set_group_list_to_redis(self, glist, cathedraid):
        current_app.ms.redis(self.lname).setex(
            "SYSGLIST" + str(cathedraid), 5, pickle.dumps(glist)
        )
        return

    def get_group_from_redis(self, id):
        cachedGlist = current_app.ms.redis(self.lname).get("GR" + str(id))
        if cachedGlist is not None:
            # current_app.logger.debug('Group from cache '+ str(id))
            glist = pickle.loads(cachedGlist)
            return glist
        return

    def set_group_to_redis(self, result):
        current_app.ms.redis(self.lname).setex(
            "GR" + str(result["group"][0]["gid"]), 5, pickle.dumps(result)
        )
        return

    def getGroup(self, lname, id):
        cc = Group(lname)

        result = cc.get_group_from_redis(id)

        if result is not None:
            return result

        conn = current_app.ms.db(lname).connect()

        stmt = text("select * from groupname where gid=:id")

        stmt = stmt.bindparams(id=id)
        query = conn.execute(stmt)

        result = {"group": [dict(zip(tuple(query.keys()), i)) for i in query.cursor]}

        if len(result["group"]) == 0:
            return None

        class_teacher = User(lname)

        if result["group"][0]["class_teacher"] is not None:
            class_teacher = class_teacher.get_user_from_sql_id(
                result["group"][0]["class_teacher"]
            )
            if class_teacher is not None:
                result["group"][0]["class_teacher"] = class_teacher.serialize()

        class_teacher2 = User(lname)

        if result["group"][0]["class_teacher2"] is not None:
            class_teacher2 = class_teacher2.get_user_from_sql_id(
                result["group"][0]["class_teacher2"]
            )
            if class_teacher2 is not None:
                result["group"][0]["class_teacher2"] = class_teacher2.serialize()

        if (
            result["group"][0]["syid"] is not None
            and result["group"][0]["f_militaryprofession"] is not None
        ):
            from LMSAPI.api.Models.Educationplan import Educationplan

            ep = Educationplan(lname)
            result["group"][0]["educationplan"] = ep.getEducationPlan(
                result["group"][0]["syid"], result["group"][0]["f_militaryprofession"]
            )
        else:
            result["group"][0]["educationplan"] = None

        if result["group"][0]["syid"] is not None:
            from LMSAPI.api.Models.Studyyear import Studyyear

            sy = Studyyear(lname)
            result["group"][0]["syid"] = sy.getStudyYear(result["group"][0]["syid"])

        if result["group"][0]["f_militaryprofession"] is not None:
            from LMSAPI.api.Models.Militaryprofession import Militaryprofession

            mp = Militaryprofession()
            result["group"][0]["f_militaryprofession"] = mp.getMilitaryProfession(
                lname, result["group"][0]["f_militaryprofession"]
            )

        cc.set_group_to_redis(result)

        return result

    def checkCathedra(self, cathedra):
        conn = current_app.ms.db(self.lname).connect()

        sql = "select cathtype from cathedras where idcathedra = :cathedra"
        stmt = text(sql)
        stmt = stmt.bindparams(cathedra=cathedra)
        query = conn.execute(stmt)

        for row in query:
            return row.cathtype

        return None

    @staticmethod
    def getGroupListByCourse(lname, faculty_id, course, chosen_date):
        conn = current_app.ms.db(lname).connect()
        where = " AND g.idfaculty IS NOT NULL"
        if faculty_id != "0":
            where += " AND g.idfaculty = :faculty_id "
        sql = """
        SELECT g.gid, g.name 
            FROM groupname g 
        JOIN group_history h ON h.gid = g.gid 
		JOIN school_year sy ON h.school_year = sy.xp_key
		    WHERE g.year = :course
		    {where}
		    AND sy.begdate < :chosen_date AND :chosen_date < sy.enddate
        """.format(
            where=where
        )

        stmt = text(sql)
        stmt = stmt.bindparams(course=course)
        stmt = stmt.bindparams(chosen_date=chosen_date)
        if faculty_id != "0":
            stmt = stmt.bindparams(faculty_id=faculty_id)
        query = conn.execute(stmt)
        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def getGroupList(self, lname, cathedra, faculty, syear, eduyear):
        c = Group(lname)

        groupList = None  # c.get_group_list_from_redis (cathedra)

        if groupList is not None:
            return groupList

        conn = current_app.ms.db(lname).connect()

        if eduyear == 0:
            andEduyear = " "
        else:
            andEduyear = " AND g.year = :eduyear "

        if cathedra == 0:
            if faculty == 0:
                sql = """
                select g.gid, h.name 
                from groupname g 
                JOIN group_history h ON h.gid = g.gid AND h.school_year = :syear  
                where g.ideducationlevel > 0  {andEduyear} order by h.name
                """.format(
                    andEduyear=andEduyear
                )

                stmt = text(sql)
                stmt = stmt.bindparams(syear=syear)

                if eduyear != 0:
                    stmt = stmt.bindparams(eduyear=eduyear)

            else:
                sql = """
                select g.gid, h.name 
				from groupname g 
                JOIN group_history h ON h.gid = g.gid AND h.school_year = :syear             
                where g.idfaculty = :faculty """

                sql = sql + andEduyear + " and g.ideducationlevel>0 order by h.name"

                stmt = text(sql)
                stmt = stmt.bindparams(syear=syear)
                stmt = stmt.bindparams(faculty=faculty)
                if eduyear != 0:
                    stmt = stmt.bindparams(eduyear=eduyear)

        else:
            if Group(lname).checkCathedra(cathedra) != 0:
                sql = """
                select g.gid, h.name 
                from groupname g 
				JOIN group_history h ON h.gid = g.gid AND h.school_year = :syear  
				where g.ideducationlevel > 0 {andEduyear} order by h.name
				""".format(
                    andEduyear=andEduyear
                )

                stmt = text(sql)
                stmt = stmt.bindparams(syear=syear)
                if eduyear != 0:
                    stmt = stmt.bindparams(eduyear=eduyear)

            else:
                sql = """select g.gid, h.name 
                     from groupname g
                     JOIN group_history h ON h.gid = g.gid AND h.school_year = :syear
                     where g.ideducationlevel>0
                     and g.idcathedra = :cathedra {andEduyear} 
                     order by h.name""".format(
                    andEduyear=andEduyear
                )
                stmt = text(sql)
                stmt = stmt.bindparams(syear=syear)
                stmt = stmt.bindparams(cathedra=cathedra)
                if eduyear != 0:
                    stmt = stmt.bindparams(eduyear=eduyear)

        # current_app.logger.debug(stmt)
        query = conn.execute(stmt)

        groupList = []

        for row in query:
            rcathedra = Group("")
            rcathedra.gid = row.gid
            rcathedra.name = row.name
            groupList.append(rcathedra)

        c.set_group_list_to_redis(groupList, cathedra)

        return groupList

    def getGroupListForTeacher(self, lname, cid, trimid):
        c = Group(lname)

        groupList = None

        if groupList is not None:
            return groupList

        conn = current_app.ms.db(lname).connect()

        sql = """select distinct  g.gid, g.name, g.* 
                    from term_weeks tw
                    left outer join nnz_weeks w on w.week_id = tw.week_id
                    left outer join nnz_schedule sh on sh.sh_var_id = w.sh_var_id
                    left outer join groupname g on sh.gid = g.gid
                    where g.gid is not null and tw.trmid = :trimid and (g.idcathedra = :cid or :cid = 0)
                    order by g.name"""
        stmt = text(sql)

        if not g.user.user_is_rasdel_admin(
            g.user.mid, "Журнал учета учебных занятий. Администратор"
        ):
            sql = """select distinct  g.gid, g.name, g.* 
                    from term_weeks tw
                    left outer join nnz_weeks w on w.week_id = tw.week_id
                    left outer join nnz_schedule sh on sh.sh_var_id = w.sh_var_id
                    left outer join groupname g on sh.gid = g.gid
                    where g.gid is not null and tw.trmid = :trimid and (g.idcathedra = :cid or :cid = 0) and '{:teacher}' && sh.teacher_mid
                    order by g.name"""
            stmt = text(sql)
            stmt = stmt.bindparams(teacher=g.user.mid)

        stmt = stmt.bindparams(cid=cid)
        stmt = stmt.bindparams(trimid=trimid)

        query = conn.execute(stmt)

        groupList = []

        for row in query:
            group = Group("")
            group.gid = row.gid
            group.name = row.name
            groupList.append(group)

        return groupList

    def getGroupListJournal(
        self,
        lname: str,
        cathedra_id: Optional[int],
        trim_id: int,
        school_year_id: int,
        faculty_id: Optional[int],
    ):
        conn = current_app.ms.db(lname).connect()
        is_admin_filter = (
            ""
            if g.user.user_is_rasdel_admin(
                g.user.mid, "Журнал учета учебных занятий. Администратор"
            )
            else " AND {teacher} = ANY(sh.teacher_mid)".format(teacher=g.user.mid)
        )

        group_filter = """
        AND EXISTS(
            SELECT
                    gid

            FROM nnz_schedule sh  
            JOIN nnz_weeks w ON w.sh_var_id = sh.sh_var_id  
            JOIN terms t ON date_add(w.wstart_date, CAST((sh.day_of_week - 1 - weekday(w.wstart_date)) || 'DAY' AS interval)) 
                                BETWEEN TO_DATE(t.term_begin,'dd.mm.yyyy') AND TO_DATE(t.term_end,'dd.mm.yyyy')  
            WHERE sh.gid = h.gid AND t.trmid = {trim_id} {admin_filter}
        )
        """.format(
            trim_id=trim_id, admin_filter=is_admin_filter
        )

        where = group_filter
        where += (
            " AND coalesce(g.idfaculty, c.faculty) = {faculty_id} ".format(
                faculty_id=faculty_id
            )
            if faculty_id
            else ""
        )
        where += (
            " AND g.idcathedra = {cathedra_id} ".format(cathedra_id=cathedra_id)
            if cathedra_id
            else ""
        )

        sql = """
        SELECT
                g.gid,
                h.name

        FROM groupname g
        LEFT JOIN cathedras c ON g.idcathedra = c.idcathedra
        JOIN group_history h ON h.gid = g.gid AND h.school_year = {school_year_id}
        WHERE g.ideducationlevel > 0 {where}
        ORDER BY (SUBSTRING(h.name, '^[0-9]+'))::int , SUBSTRING(h.name, '[^0-9_].*$')
        """.format(
            school_year_id=school_year_id,
            where=where,
        )

        stmt = text(sql)
        query = conn.execute(stmt)
        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def getAllGroups(self, lname):
        c = Group(lname)

        groupList = None

        if groupList is not None:
            return groupList

        conn = current_app.ms.db(lname).connect()

        sql = """select distinct  g.gid, g.name, g.* 
				from groupname g
				where g.gid is not null 
				order by g.name"""
        stmt = text(sql)

        query = conn.execute(stmt)

        groupList = []

        for row in query:
            group = Group("")
            group.gid = row.gid
            group.name = row.name
            groupList.append(group)

        return groupList

    def getGroupStudents(self, lname, gid):
        c = Group(lname)

        groupList = None

        if groupList is not None:
            return groupList

        conn = current_app.ms.db(lname).connect()

        sql = """select 
        p.mid, lastname, firstname, patronymic
        from people p
        left outer join groupuser gu on p.mid = gu.mid and gu.cid = -1           
        where gu.gid = :gid
        order by lastname, firstname, patronymic"""
        stmt = text(sql)
        stmt = stmt.bindparams(gid=gid)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def get_groups_by_role(self, lname, xp_key, idfaculty, idcathedra):
        conn = current_app.ms.db(lname).connect()

        sql = """
            SELECT distinct gn.name, gn.gid
            FROM permission2mid p2m  
            JOIN permission_groups pg ON pg.pmid = p2m.pmid 
            JOIN vw_divisions vd ON vd.id = ANY(pg.sdid) 
            JOIN groupname gn ON gn.idcathedra | 536870912 = vd.id  
                              OR gn.idfaculty = vd.id 
                              OR gn.sdid | 268435456 = vd.id 
            JOIN group_history h ON h.gid = gn.gid AND (h.school_year = :xp_key OR 0 = :xp_key)
            WHERE p2m.mid = :mid AND gn.cid = -1 and (gn.idfaculty = :idfaculty OR 0 = :idfaculty) and (gn.idcathedra = :idcathedra OR 0 = :idcathedra)
            """
        stmt = text(sql)
        stmt = stmt.bindparams(mid=g.user.mid)
        stmt = stmt.bindparams(xp_key=xp_key)
        stmt = stmt.bindparams(idfaculty=idfaculty)
        stmt = stmt.bindparams(idcathedra=idcathedra)
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def get_groups_by_admin(self, lname, xp_key, idfaculty, idcathedra):
        conn = current_app.ms.db(lname).connect()

        sql = """
            SELECT distinct gn.name, gn.gid
            FROM groupname gn
            JOIN group_history h ON h.gid = gn.gid AND (h.school_year = :xp_key OR 0 = :xp_key)
            WHERE gn.cid = -1 and (gn.idfaculty = :idfaculty OR 0 = :idfaculty) and (gn.idcathedra = :idcathedra OR 0 = :idcathedra)
            """
        stmt = text(sql)
        stmt = stmt.bindparams(xp_key=xp_key)
        stmt = stmt.bindparams(idfaculty=idfaculty)
        stmt = stmt.bindparams(idcathedra=idcathedra)
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]
