# LMSAPI/API/Faculty.py

from sqlalchemy import create_engine
from sqlalchemy.sql import text
from LMSAPI.api.Models.User import User
from flask import current_app
import pickle

class Message:

    def __init__(self,cname):
        self.idfaculty=1
        self.faculty=""
        self.lname = cname

    def serialize(self):  
        return {           
        'idfaculty': self.idfaculty,
        'faculty': self.faculty
        }

    def get_faculty_list_from_redis(self):
        cachedFlist = current_app.ms.redis(self.lname).get("SYSFLIST")
        if cachedFlist is not None:
            flist = pickle.loads(cachedFlist)
            return flist
        return

    def set_faculty_list_to_redis(self,flist):
        current_app.ms.redis(self.lname).setex("SYSFLIST",5,pickle.dumps(flist))
        return

    def get_faculty_from_redis(self,id):
        cachedFlist = current_app.ms.redis(self.lname).get("FA" + str(id))
        if cachedFlist is not None:
            flist = pickle.loads(cachedFlist)
            return flist
        return

    def set_faculty_to_redis(self,result):
        current_app.ms.redis(self.lname).setex("FA" + str(result['faculty'][0]['idfaculty']),5,pickle.dumps(result))
        return



    def  getChatList(self,mid):

        conn = current_app.ms.db(self.lname).connect()

        query = """
select
  mc.ID,
  mc.From,
  mc.To,
  mc.Title,
  if(mc.To=:mid,mc.From,mc.To) as Photoid,
  to_char(mc.Created,'DD.MM.YYYY HH:mm') as Created ,
  to_char(mc.Updated,'DD.MM.YYYY HH:mm') as Updated , 
  if(:mid= mc.from, mc.from_last_viewed, mc.to_last_viewed) LastViewed,
  (select count(mm.id) from messages_messages mm where mm.chat = mc.id) Count,
  (select count(mm.id) from messages_messages mm where mm.chat = mc.id and mm.id >  if(:mid= mc.from, mc.from_last_viewed, mc.to_last_viewed) )  as CountNew,
  cast(concat(trim(pf.LastName), ' ', trim(pf.FirstName), ' ', trim(pf.Patronymic)) as text) as  FromName,
  cast(concat(trim(pt.LastName), ' ', trim(pt.FirstName), ' ', trim(pt.Patronymic)) as text) as ToName,
  if(:mid = mc.from, mc.from_hide, mc.to_hide) Hided
from messages_chats mc
  inner join people pf on pf.mid = mc.from
  inner join people pt on pt.mid = mc.to
where
  ((mc.from = :mid) or (mc.to = :mid)) and
  (:mid = 1 or if(mc.from = :mid, mc.from_hide, mc.to_hide) = 0)
  order by updated desc;
--order by if(CountNew>0,0,1), updated desc;
 """
        stmt = text(query)
        stmt = stmt.bindparams(mid=mid)    
       
    
        query = conn.execute(stmt)
                
        result = {'chat': [dict(zip(tuple (query.keys()) ,i)) for i in query.cursor]}
             
        return result


    def  getMessageCount(self,mid,all):

        conn = current_app.ms.db(self.lname).connect()

        query = """
select
  sum(CountNew) as CountNew
from (
  select
    if(:mid = mc.from , mc.from_last_viewed, mc.to_last_viewed) LastViewed,
    (select count(mm.id) from messages_messages mm where mm.chat = mc.id and mm.id > if(:mid= mc.from , mc.from_last_viewed, mc.to_last_viewed) ) CountNew
  from messages_chats mc
  where
  ((mc.from = :mid ) or (mc.to = :mid )) and
  
  (:all = 1 or if(mc.from = :mid  , mc.from_hide, mc.to_hide) = 0)
  order by updated desc
) t;
        """
        stmt = text(query)
        stmt = stmt.bindparams(mid=mid)    
        stmt = stmt.bindparams(all=all)    
        query = conn.execute(stmt)
                
        newCount = 0
        for row in query:
            if(row.countnew != None):
                newCount  = int(row.countnew)
                 
        if newCount is None:
            return 0

        return newCount


    def  getChatInfo(self,mid,chatid):

        conn = current_app.ms.db(self.lname).connect()

        query = """
  select
  mc.ID,
  mc.From,
 -- mc.From_Type,
  mc.To,
  --mc.To_Type,
  mc.Title,
  if(mc.To=:mid,mc.From,mc.To) as Photoid,
  to_char(mc.Created,'DD.MM.YYYY HH:mm') as Created ,
  to_char(mc.Updated,'DD.MM.YYYY HH:mm') as Updated , 
  mc.Type,
    if(:mid = mc.from , mc.from_last_viewed, mc.to_last_viewed) LastViewed,
  (select count(mm.id) from messages_messages mm where mm.chat = mc.id) Count,
  (select count(mm.id) from messages_messages mm where mm.chat = mc.id and mm.id >  if(:mid = mc.from , mc.from_last_viewed, mc.to_last_viewed)) CountNew,
  cast(concat(trim(pf.LastName), ' ', trim(pf.FirstName), ' ', trim(pf.Patronymic))
    as text) FromName,
  cast(concat(trim(pt.LastName), ' ', trim(pt.FirstName), ' ', trim(pt.Patronymic))
    as text) ToName
from messages_chats mc
  inner join people pf on pf.mid = mc.from
  inner join people pt on pt.mid = mc.to
where
  mc.id = :chatid and
  ((mc.from = :mid ) or (mc.to = :mid )) 
        """
        stmt = text(query)
        stmt = stmt.bindparams(mid=mid)    
        stmt = stmt.bindparams(chatid=chatid)    
        query = conn.execute(stmt)
                
        result = {'chatinfo': [dict(zip(tuple (query.keys()) ,i)) for i in query.cursor]}
       
        return result

    def  getChat(self,mid,chatid):

        conn = current_app.ms.db(self.lname).connect()

        query = """
  select
  mm.ID,
  mm.Parent,
  mm.From,
  mm.Message,
  to_char(mc.Created,'DD.MM.YYYY HH:mm') as Created ,
  cast(concat(trim(pf.LastName), ' ', trim(pf.FirstName), ' ', trim(pf.Patronymic))as text) as FromName,
  if(mm.id > if(:mid = mc.from, mc.from_last_viewed, mc.to_last_viewed), 1, 0)  as New
from messages_messages mm
  inner join messages_chats mc on mc.id = mm.chat
  inner join people pf on pf.mid = mm.from
where
  mc.id = :chatid and
  ((mc.from = :mid ) or (mc.to = :mid ))
order by mm.parent, mm.created;

        """
        stmt = text(query)
        stmt = stmt.bindparams(mid=mid)    
        stmt = stmt.bindparams(chatid=chatid)    
        query = conn.execute(stmt)                
        result =[dict(zip(tuple (query.keys()) ,i)) for i in query.cursor]
        return result

    def  updateMessagesChatFrom(self,mid,chatid,messageid):
        conn = current_app.ms.db(self.lname).connect()
        query = """
update messages_chats 
set
  from_last_viewed = :messageid
where
  id = :chatid and  "from" = :mid ;
        """

        stmt = text(query)
        stmt = stmt.bindparams(mid=mid)    
        stmt = stmt.bindparams(chatid=chatid)    
        stmt = stmt.bindparams(messageid=messageid)    
        query = conn.execute(stmt)
        
    
    def  updateMessagesChatTo(self,mid,chatid,messageid):
        conn = current_app.ms.db(self.lname).connect()
        query = """
        update messages_chats  mc
set
  to_last_viewed = :messageid
  where

  id = :chatid and "to" = :mid ;
        """

        stmt = text(query)
        stmt = stmt.bindparams(mid=mid)    
        stmt = stmt.bindparams(chatid=chatid)    
        stmt = stmt.bindparams(messageid=messageid)    
        query = conn.execute(stmt)
    
    def  searchUsers(self,parameter):
        conn = current_app.ms.db(self.lname).connect()

        parameter = "%" + parameter + "%"

        query = """
select mid,lastname,firstname,patronymic 
from people 
where login > '' and (concat(lastname, ' ',  firstname, ' ', patronymic) ilike :parameter
or concat(firstname, ' ', patronymic, ' ', lastname) ilike :parameter)
order by lastname,patronymic,firstname
        """
        stmt = text(query)
        stmt = stmt.bindparams(parameter=parameter)    
       
        query = conn.execute(stmt)                
        result = {'people': [dict(zip(tuple (query.keys()) ,i)) for i in query.cursor]}       
        return result

    def newChat(self,midfrom,midto,title,message):
        conn = current_app.ms.db(self.lname).connect()
        query = """
insert into messages_chats  ("from", "to",  title,type,created,updated)
values (:midfrom, :midto, :title,0,now(),now());
        """
        stmt = text(query)
        stmt = stmt.bindparams(midto=midto)    
        stmt = stmt.bindparams(midfrom=midfrom)  
        stmt = stmt.bindparams(title=title)    
        query = conn.execute(stmt)

        sql = "SELECT currval(pg_get_serial_sequence('messages_chats','id'))"
        stmt = text(sql)        
        query = conn.execute(stmt)
                
        newId = 0
        for row in query:
            newId     = row.currval
            
        if newId is None:
            return -1

        if message != None and message != "":
            messageid = self.newMessage(newId,0,midfrom,message)        
            self.updateMessagesChatFrom(midfrom,newId,messageid)    

        return newId


    def deleteChat(self,midfrom,chatid):

        conn = current_app.ms.db(self.lname).connect()
        
        # Check permission
        query = """select id from messages_chats where "from" = :midfrom and id=:chatid"""
        stmt = text(query)
        stmt = stmt.bindparams(midfrom=midfrom)  
        stmt = stmt.bindparams(chatid=chatid)    
        query = conn.execute(stmt)

        chat = 0

        for row in query:
            chat  = int(row.id)

        # We do not have right to do this
        if chat == 0:
            return False

        query = """
delete from messages_messages where chat =:chatid
        """
        stmt = text(query)        
        stmt = stmt.bindparams(chatid=chatid)    
        query = conn.execute(stmt)

        query = """
delete from messages_chats where "from" = :midfrom and id=:chatid
        """
        stmt = text(query)
        stmt = stmt.bindparams(midfrom=midfrom)  
        stmt = stmt.bindparams(chatid=chatid)    
        query = conn.execute(stmt)
        
        return True



    def newMessage(self,chatid,parentid,midfrom,message):
        conn = current_app.ms.db(self.lname).connect()
        query = """
insert into messages_messages (chat, parent, "from", message,created)
values (:chatid, :parentid, :midfrom, :message,now());
        """
        stmt = text(query)
        stmt = stmt.bindparams(chatid=chatid)    
        stmt = stmt.bindparams(parentid=parentid)    
        stmt = stmt.bindparams(midfrom=midfrom)  
        stmt = stmt.bindparams(message=message)    
        query = conn.execute(stmt)
        sql = "SELECT currval(pg_get_serial_sequence('messages_messages','id'))"
        stmt = text(sql)        
        query = conn.execute(stmt)
                
        newId = 0
        for row in query:
            newId     = row.currval
            
        if newId is None:
            return -1
        return newId


    def deleteMessage(self,messageid, midfrom):

        conn = current_app.ms.db(self.lname).connect()
        
        # Check permission
        query = """select id,parent from messages_messages where "from" = :midfrom and id=:messageid"""
        stmt = text(query)
        stmt = stmt.bindparams(midfrom=midfrom)  
        stmt = stmt.bindparams(messageid=messageid)    
        query = conn.execute(stmt)

        msg = 0
        parent = 0
        for row in query:
            msg  = int(row.id)
            parent = int (row.parent)

        # We do not have right to do this
        if msg == 0:
            return False

        query = """
        delete from messages_messages where id =:messageid
        """
        stmt = text(query)        
        stmt = stmt.bindparams(messageid=messageid)    
        query = conn.execute(stmt)

        query = """
        update messages_messages set parent=:newparent where parent =:messageid
        """
        stmt = text(query)        
        stmt = stmt.bindparams(newparent=parent)    
        stmt = stmt.bindparams(messageid=messageid)    
        query = conn.execute(stmt)

        return True
