# coding=UTF-8
# LMSAPI/API/Journal.py
from flask import current_app, g
import pickle

from sqlalchemy import text


class QuestionSet:
    def get_questionset_all(self, lname, cid, private, oid):
        conn = current_app.ms.db(lname).connect()
        where = ""
        if cid:
            where += " and cid = :cid "
        if private is not None:
            where += " and private = :private "
        if oid:
            where += " and oid = :oid "

        sql = """
            select * from questionset
            where 1=1
            {where}
            """.format(
            where=where
        )

        stmt = text(sql)
        if cid:
            stmt = stmt.bindparams(cid=cid)
        if private is not None:
            stmt = stmt.bindparams(private=private)
        if oid:
            stmt = stmt.bindparams(oid=oid)

        query = conn.execute(stmt)
        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def get_question_and_answer_by_test(self, lname, id, teacher):
        conn = current_app.ms.db(lname).connect()

        correct = ", 'correct', ea.correct"
        if teacher == False:
            correct = ""

        sql = """
            WITH questions_with_children AS (
                SELECT 
                    eq.id as question_id,
                    eq.question_type as question_type,
                    eq.cid as cid,
                    eq.oid as oid,
                    eq.tlid as tlid,
                    eq.question_text as question_text,
                    eq.q_num as q_num,
                    eq.weight as weight,
                    eq.id_parent as id_parent,
                    eq.status as status,
                    eq.eyid as eyid,
                    eq.answer_id as answer_id,
                    eq.worktime as worktime
                FROM 
                    questionset q
                JOIN questionset_content qc ON qc.id_set = q.id
                JOIN enroll_questions eq ON eq.id = qc.id_question
                WHERE 
                    q.id = :id
                    AND eq.question_type <> 6  -- Берём все вопросы, кроме тех, где question_type = 6
                    AND eq.question_type <> 5  -- Не берём вопросы с question_type = 5
            )
            SELECT 
                qwc.question_id,
                qwc.question_type,
                qwc.cid,
                qwc.oid,
                qwc.tlid,
                qwc.question_text,
                qwc.q_num,
                qwc.weight,
                qwc.id_parent,
                qwc.status,
                qwc.eyid,
                qwc.answer_id,
                qwc.worktime,
                json_agg(
                    json_build_object(
                        'id', ea.id,
                        'answer_text', ea.answer_text,
                        'answer_group', ea.answer_group,  
                        'answer_length', ea.answer_length,
                        'answer_pos', ea.answer_pos,
                        'answer_html', ea.answer_html,
                        'sel_start', ea.sel_start,
                        'sel_len', ea.sel_len,
                        'value_num', ea.value_num
                        {correct}
                    )
                ) AS answers
            FROM 
                questions_with_children qwc
            LEFT JOIN 
                enroll_answers ea ON ea.question = qwc.question_id
            GROUP BY 
                qwc.question_id, qwc.question_type, qwc.cid, qwc.oid, qwc.tlid, qwc.question_text, 
                qwc.q_num, qwc.weight, qwc.id_parent, qwc.status, qwc.eyid, qwc.answer_id, qwc.worktime
            ORDER BY 
                qwc.question_type;
            """.format(
            correct=correct
        )

        stmt = text(sql)
        stmt = stmt.bindparams(id=id)

        query = conn.execute(stmt)
        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def get_questionset_by_id(self, lname, id):
        conn = current_app.ms.db(lname).connect()

        sql = """
            select * from questionset
            where id = :id
            """
        stmt = text(sql)
        stmt = stmt.bindparams(id=id)
        query = conn.execute(stmt)

        result = query.fetchone()

        if result is None:
            return None

        result_dict = dict(zip(query.keys(), result))
        return result_dict

    def create_questionset(self, lname, data):
        conn = current_app.ms.db(lname).connect()

        sql = """ 
            insert into questionset(name, cid, private, description, oid) 
            values (:name, :cid, :private, :description, :oid) 
            returning id 
            """
        stmt = text(sql)
        stmt = stmt.bindparams(
            name=data.get("name"),
            cid=data.get("cid"),
            private=data.get("private"),
            description=data.get("description"),
            oid=data.get("oid"),
        )
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def update_questionset(self, lname, id, data):
        conn = current_app.ms.db(lname).connect()

        # Список для хранения частей SET выражения и параметров
        set_clauses = []
        params = {"id": id}

        # Добавляем поля в SET выражение, если они есть в данных
        if data.get("cid") is not None:
            set_clauses.append("cid = :cid")
            params["cid"] = data.get("cid")
        if data.get("private") is not None:
            set_clauses.append("private = :private")
            params["private"] = data.get("private")
        if data.get("oid") is not None:
            set_clauses.append("oid = :oid")
            params["oid"] = data.get("oid")
        if data.get("name") is not None:
            set_clauses.append("name = :name")
            params["name"] = data.get("name")
        if data.get("description") is not None:
            set_clauses.append("description = :description")
            params["description"] = data.get("description")

        # Проверяем, есть ли поля для обновления
        if not set_clauses:
            raise ValueError("No fields to update")

        # Формируем SQL запрос
        set_clause = ", ".join(set_clauses)
        sql = """
            UPDATE questionset
            SET {set_clause}
            WHERE id = :id
            RETURNING id
        """.format(
            set_clause=set_clause
        )

        stmt = text(sql).bindparams(**params)
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def delete_questionset(self, lname, id):
        conn = current_app.ms.db(lname).connect()
        sql = """
            DELETE FROM questionset
            WHERE id = :id
            RETURNING id
        """

        stmt = text(sql).bindparams(id=id)
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]


class QuestionSetContent:
    def get_questionset_content_all(self, lname, id_question, id_set):
        conn = current_app.ms.db(lname).connect()
        where = ""
        if id_question:
            where += " and id_question = :id_question "
        if id_set:
            where += " and id_set = :id_set "

        sql = """
            select * from questionset_content
            where 1=1
            {where}
            """.format(
            where=where
        )

        stmt = text(sql)
        if id_question:
            stmt = stmt.bindparams(id_question=id_question)
        if id_set:
            stmt = stmt.bindparams(id_set=id_set)

        query = conn.execute(stmt)
        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def create_questionset_content(self, lname, data):
        conn = current_app.ms.db(lname).connect()

        sql = """ 
            insert into questionset_content(id_set, id_question, sortnum) 
            values (:id_set, :id_question, :sortnum) 
            returning id_set 
            """
        stmt = text(sql)
        stmt = stmt.bindparams(
            id_set=data.get("id_set"),
            id_question=data.get("id_question"),
            sortnum=data.get("sortnum"),
        )
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

    def delete_questionset_content(self, lname, id_question, id_set):
        conn = current_app.ms.db(lname).connect()
        sql = """
            DELETE FROM questionset_content
            WHERE id_set = :id_set and id_question = :id_question
            RETURNING id_set
        """

        stmt = text(sql).bindparams(id_question=id_question, id_set=id_set)
        query = conn.execute(stmt)

        return [dict(zip(tuple(query.keys()), i)) for i in query.cursor]
