
from sqlalchemy import create_engine
from sqlalchemy.sql import text
from flask import current_app
import pickle
from LMSAPI.api.Models.File import File
class StudentProject:

    def __init__(self,cname):
        self.lname = cname

        self.id = None
        self.type = None
        self.student = None
        self.teacher = None
        self.cid = None
        self.issuedate = None
        self.duedate = None
        self.grade = None
        self.task = None
        self.result = None
        self.status = None
 
        self.taskfiles = []
        self.resultfiles = []
        self.studentreviews = []
       
    def getStudentProjects(self,termid, mid, tid, cid, isTeacher): # cid - 0 , all projects , termid never 0

        conn = current_app.ms.db(self.lname).connect()
        
        sql = """select xp_student_project.*, 
        cast(concat(trim(te.LastName), ' ', trim(te.FirstName), ' ', trim(te.Patronymic))as text) as teachername,
        cast(concat(trim(ts.LastName), ' ', trim(ts.FirstName), ' ', trim(ts.Patronymic))as text) as studentname,
        c.title as coursetitle, c.alias as coursealias
        from xp_student_project 
        inner join people te on te.mid = xp_student_project.teacher
        inner join people ts on ts.mid = xp_student_project.student
        inner join courses c on c.cid = xp_student_project.cid
        LEFT OUTER JOIN terms t ON trmid = :termid
        WHERE xp_student_project.issuedate BETWEEN str_to_date(t.term_begin, '%d.%m.%Y') AND str_to_date(t.term_end, '%d.%m.%Y') """

        mid = int(mid)

        if isTeacher == False or mid != 0:
            sql = sql + " and student=:mid "

        tid = int(tid)
        cid = int(cid)

        if tid != 0:
            current_app.logger.debug(tid)
            sql = sql + " and teacher=:tid"

        if cid != 0:
            current_app.logger.debug(cid)
            sql = sql + " and xp_student_project.cid=:cid"

        stmt = text(sql)
        #current_app.logger.debug(stmt)

        stmt = stmt.bindparams(termid=termid)    

        if mid != 0:
            stmt = stmt.bindparams(mid=mid)    

        if tid != 0:
            stmt = stmt.bindparams(tid=tid)    

        if cid != 0:
            stmt = stmt.bindparams(cid=cid)    

        query = conn.execute(stmt)

 
        studentprojects = [dict(zip(tuple (query.keys()) ,i)) for i in query.cursor]

        return studentprojects
    

    def getStudentProject(self,id):

        conn = current_app.ms.db(self.lname).connect()
        
        sql = """select xp_student_project.* ,
        cast(concat(trim(te.LastName), ' ', trim(te.FirstName), ' ', trim(te.Patronymic))as text) as teachername,
        cast(concat(trim(ts.LastName), ' ', trim(ts.FirstName), ' ', trim(ts.Patronymic))as text) as studentname,
        c.title as coursetitle, c.alias as coursealias
        from xp_student_project 
        inner join people te on te.mid = xp_student_project.teacher
        inner join people ts on ts.mid = xp_student_project.student
        inner join courses c on c.cid = xp_student_project.cid
        where xp_key=:id
        
        """
        stmt = text(sql)
        stmt = stmt.bindparams(id=id)   
        query = conn.execute(stmt)

        
        for row in query:            
            self.id = row.xp_key
            self.type = row.type
            self.student = row.student
            self.studentname = row.studentname
            self.teacher = row.teacher
            self.teachername = row.teachername
            self.cid = row.cid
            self.issuedate = row.issuedate
            self.duedate = row.duedate
            self.grade =row.grade
            self.task = row.task
            self.result = row.result
            self.status = row.status
            self.coursetitle = row.coursetitle
            self.coursealias = row.coursealias
            
        if self.id == None:
            return False

        return True


    def getStudentProjectData(self,id):

        if self.getStudentProject(id) == False:
            return None

        self.taskfiles = File(self.lname).getStudentProjectTaskDirectoryFiles(self.id)
        self.resultfiles= File(self.lname).getStudentProjectResultDirectoryFiles(self.id)

        self.getStudentProjectReviews()

        for review in self.studentreviews:
            review['reviewfiles'] =  File(self.lname).getStudentProjectReviewDirectoryFiles(self.id, review['xp_key'])

        data = { 
            'taskfiles': self.taskfiles,
            'resultfiles': self.resultfiles,
            'reviews': self.studentreviews,
            'id': self.id,
            'type':  self.type,
            'studentmid': self.student,
            'studentname': self.studentname,
            'teachermid': self.teacher,
            'teachername': self.teachername,
            'cid': self.cid,
            'issuedate': self.issuedate,
            'duedate': self.duedate,
            'grade': self.grade,
            'task': self.task,
            'result': self.result,
            'status': self.status,
            'coursealias': self.coursealias,
            'coursetitle': self.coursetitle      
        }
        return data

    def editStudentProject(self):
        conn = current_app.ms.db(self.lname).connect()
        
        sql = """UPDATE public.xp_student_project
	SET  type=:type, student=:student, teacher=:teacher, cid=:cid, issuedate=:issuedate, 
    duedate=:duedate, grade=:grade, task=:task, result=:result, status=:status
	WHERE xp_key=:id;"""        
        stmt = text(sql)
        
        stmt = stmt.bindparams(id=self.id)   
        stmt = stmt.bindparams(type=self.type)   
        stmt = stmt.bindparams(student=self.student)   
        stmt = stmt.bindparams(teacher=self.teacher)   
        stmt = stmt.bindparams(cid=self.cid)
        stmt = stmt.bindparams(issuedate=self.issuedate)
        stmt = stmt.bindparams(duedate=self.duedate)
        stmt = stmt.bindparams(grade=self.grade)
        stmt = stmt.bindparams(task=self.task)
        stmt = stmt.bindparams(result=self.result)
        stmt = stmt.bindparams(status=self.status)                     
        conn.execute(stmt)

        return True


    def deleteStudentProject(self):
        conn = current_app.ms.db(self.lname).connect()
        
        sql = """delete from xp_student_project 
	WHERE xp_key=:id;"""        
        stmt = text(sql)    
        stmt = stmt.bindparams(id=self.id)   
        conn.execute(stmt)

        return True

    
    def createStudentProject(self):

        conn = current_app.ms.db(self.lname).connect()
        
        sql = """INSERT INTO public.xp_student_project(
	 type, student, teacher, cid, issuedate, duedate, grade, task, result, status)
	VALUES (:type, :student, :teacher, :cid, :issuedate, :duedate, :grade, :task, :result, :status);"""        
        stmt = text(sql)
                
        stmt = stmt.bindparams(type=self.type)   
        stmt = stmt.bindparams(student=self.student)   
        stmt = stmt.bindparams(teacher=self.teacher)   
        stmt = stmt.bindparams(cid=self.cid)
        stmt = stmt.bindparams(issuedate=self.issuedate)
        stmt = stmt.bindparams(duedate=self.duedate)
        stmt = stmt.bindparams(grade=self.grade)
        stmt = stmt.bindparams(task=self.task)
        stmt = stmt.bindparams(result=self.result)
        stmt = stmt.bindparams(status=self.status)                     
        conn.execute(stmt)

        sql = "SELECT currval(pg_get_serial_sequence('xp_student_project','xp_key'))"
        stmt = text(sql)        
        query = conn.execute(stmt)                
        newId = 0
        for row in query:
            newId     = row.currval

        return newId


    def getStudentProjectReviews(self):

        if self.id == None:
            return False
        
        conn = current_app.ms.db(self.lname).connect()
        
        sql = """select xp_student_project_review.*,
                        cast(concat(trim(te.LastName), ' ', trim(te.FirstName), ' ', trim(te.Patronymic))as text) as teachername
         from xp_student_project_review 
         inner join people te on te.mid = xp_student_project_review.mid
         where project=:project"""
        stmt = text(sql)
        stmt = stmt.bindparams(project=self.id)   
        query = conn.execute(stmt)
        self.studentreviews = [dict(zip(tuple (query.keys()) ,i)) for i in query.cursor] 

        return True       


    def getStudentProjectReview(self,rid):

        conn = current_app.ms.db(self.lname).connect()
        
        sql = """select *
         from xp_student_project_review 
         inner join people te on te.mid = xp_student_project_review.mid
         where xp_student_project_review.xp_key=:rid"""
        stmt = text(sql)
        stmt = stmt.bindparams(rid=rid)   
        query = conn.execute(stmt)

        for row in query:                                    
            self.srproject = row.project
            self.srmid = row.mid
            self.srid = row.xp_key
            self.srreview = row.review
            self.srreviewdate = row.reviewdate
            return True
        
        return False   

    def createStudentProjectReview(self, mid, review):

        conn = current_app.ms.db(self.lname).connect()

        sql = """INSERT INTO public.xp_student_project_review(
	 project, mid, review, reviewdate)
	VALUES ( :project, :mid, :review, now());"""        
        stmt = text(sql)
                
        stmt = stmt.bindparams(project=self.id)   
        stmt = stmt.bindparams(mid=mid)   
        stmt = stmt.bindparams(review=review)   
        
        conn.execute(stmt)

        sql = "SELECT currval(pg_get_serial_sequence('xp_student_project_review','xp_key'))"
        stmt = text(sql)        
        query = conn.execute(stmt)                
        newId = 0
        for row in query:
            newId     = row.currval

        return newId
        

    def editStudentProjectReview(self, review):

        conn = current_app.ms.db(self.lname).connect()

        sql = """UPDATE xp_student_project_review SET review=:review, reviewdate=now()
	WHERE xp_key=:id"""        
        stmt = text(sql)
                
        stmt = stmt.bindparams(id=self.srid)   
        stmt = stmt.bindparams(review=review)   
        conn.execute(stmt)


        return True

    def deleteStudentProjectReview(self,id):

        
        conn = current_app.ms.db(self.lname).connect()
        sql = """DELETE FROM public.xp_student_project_review
	WHERE xp_key=:id"""        
        stmt = text(sql)
                
        stmt = stmt.bindparams(id=id)           
        conn.execute(stmt)

        return True
    
