import os
import json

from flask import jsonify, request
from flask import Blueprint
from flask_cors import CORS

from LMSAPI.api.Views.TokenAPI import auth
from LMSAPI.api.Models.Dashboard import Dashboard

dashboard_api = Blueprint("dashboard_api", __name__)
CORS(dashboard_api)


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/structure", methods=["GET"])
def get_structure(lname):
    structure_json_path = "./static/dashboard/structure.json"
    if os.path.exists(structure_json_path):
        with open(structure_json_path, "r") as json_file:
            data = json_file.read()
            return jsonify(json.loads(data))
    else:
        return jsonify({"error": "Файл не найден"})


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/university", methods=["GET"])
def get_university(lname):
    university_json_path = "./static/dashboard/university.json"
    if os.path.exists(university_json_path):
        with open(university_json_path, "r") as json_file:
            data = json_file.read()
            return jsonify(json.loads(data))
    else:
        return jsonify({"error": "Файл не найден"}), 404


@dashboard_api.route(
    "/lms/api/v1.0/<lname>/dashboard/training_quality", methods=["GET"]
)
def get_training_quality(lname):
    training_quality = {
        "speciality": [
            {"name": "Военные специальности командного профиля", "count": 166},
            {"name": "Военные специальности инженерного профиля", "count": 209},
            {"name": "Другие военные специальности", "count": 22},
        ],
        "honors_degree": [
            {"name": "Завершили обучение с медалью", "perc": 3.3},
            {"name": "Завершили обучение с дипломо с отличием", "perc": 14.2},
        ],
        "fgos": [
            {
                "created_by": [
                    {"name": "ФГОС разработанные вузами МО РФ", "count": 63},
                    {"name": "ФГОС разработанные Минобрнауки России", "count": 46},
                ],
                "areas_of_training": [
                    {"name": "ФГОС подготовки слушателей", "count": 14},
                    {"name": "ФГОС подготовки курсантов с ВО", "count": 60},
                    {"name": "ФГОС подготовки курсантов с СПО", "count": 35},
                ],
            }
        ],
        "avg_mark": [
            {"name": "Государственный междисциплинарный экзамен", "score": 4.35},
            {"name": "Защита ВКР", "score": 4.42},
            {"name": "Государственный экзамен по физической подготовке", "score": 4.41},
        ],
    }
    return jsonify(training_quality)


@dashboard_api.route(
    "/lms/api/v1.0/<lname>/dashboard/information_support", methods=["GET"]
)
def get_information_support(lname):
    information_support = {
        "electronic_textbook": {"base": 14381, "others": 113, "count": 14494},
        "pc": {
            "year_stat_perc": {
                "2017": 7.2,
                "2018": 14.1,
                "2019": 21.0,
                "2020": 27.9,
                "2021": 41.5,
                "2022": [{"jun": 65.1, "aug": 78.9, "sep": 97.0}],
            }
        },
        "library": {
            "users": 84676,
            "ir_loading": {
                "2021": [{"dec": 100}],
                "2022": [{"apr": 1500, "jun": 3000, "aug": 4740}],
            },
        },
    }
    return jsonify(information_support)


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/staffing", methods=["GET"])
def get_staffing(lname):
    staffing = {
        "teachers_staffing_perc": {"total": 89, "military": 89, "civil": 90},
        "professor_combat_exp_perc": 32.2,
        "additional_edu": {
            "prof_retraining": 89,
            "advanced_training": 32.7,
            "traineeship": 10.7,
        },
        "scientists": {
            "perc": 71,
            "ratio": [
                {"name": "Доктора наук", "perc": 13},
                {"name": "Кандидаты наук", "perc": 58},
                {"name": "Высшее образование", "perc": 29},
            ],
        },
        "dissertation_defense_perc": 77,
    }
    return jsonify(staffing)


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/vka_performance", methods=["GET"])
def get_vka_perfomance(lname):
    perf = {
        "pps_stat": {
            "military_civil_perc": {"military": 49, "civil": 51},
            "scientific_perc": [
                {"name": "Доктора наук", "perc": 12.2},
                {"name": "Кандидаты наук", "perc": 59.9},
                {"name": "Профессора", "perc": 7.2},
                {"name": "Доценты", "perc": 33.2},
                {"name": "Профессора и доценты без ученой степени", "perc": 3},
                {"name": "Не имеют научной степени, ученого звания", "perc": 24.9},
            ],
        },
        "pps_age": [
            {"name": "До 30 лет", "perc": 9.7},
            {"name": "От 36 до 45 лет", "perc": 29.9},
            {"name": "От 56 до 55 лет", "perc": 33.2},
            {"name": "От 56 до 65 лет", "perc": 15.1},
            {"name": "Старше 65", "perc": 12.1},
        ],
        "final_marks": {
            "avg_interdisciplinary_mark": 4.80,
            "avg_graduation_mark": 4.92,
            "avg_state_exam_mark": 4.41,
        },
        "electronic_env": {
            "el_textbook_perc": 90,
            "video_lectures_perc": 76,
            "el_test_perc": 15,
            "local_network_perc": 76,
        },
        "staff_adjunct_doctor": {
            "doctor_perc": 94,
            "full_time_adjunct_perc": 99.1,
            "extramural_adjunct_perc": 58,
        },
        "progress_dynamics": [
            {"month": "sep", "avg_score": 4.8},
            {"month": "oct", "avg_score": 4.6},
            {"month": "nov", "avg_score": 4.5},
            {"month": "dec", "avg_score": 4.1},
            {"month": "jan", "avg_score": 4.5},
            {"month": "feb", "avg_score": 4.8},
            {"month": "mar", "avg_score": 4.7},
            {"month": "apr", "avg_score": 4.6},
            {"month": "may", "avg_score": 4.2},
            {"month": "jun", "avg_score": 4.7},
            {"month": "jul", "avg_score": 4.5},
            {"month": "aug", "avg_score": 4.4},
        ],
        "absence_reason": [
            {"name": "Отпуск", "perc": 91},
            {"name": "Прочее", "perc": 3},
            {"name": "Болезнь", "perc": 3},
            {"name": "Наряд", "perc": 2},
            {"name": "Командировка", "perc": 1},
        ],
    }
    return jsonify(perf)


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/main_pps_stat", methods=["GET"])
def get_main_pps_stat(lname):
    pps_stat = Dashboard.get_pps_main_stat(lname)
    return jsonify(pps_stat)


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/age_stat", methods=["GET"])
def age_stat(lname):
    stat = Dashboard.get_age_stat(lname)
    return jsonify(stat)


@dashboard_api.route("/lms/api/v1.0/<lname>/dashboard/university_stat", methods=["GET"])
def get_university_stat(lname):
    university_stat = {}
    main_stat_path = "./static/dashboard/main_stat.json"
    main_data = {}
    if os.path.exists(main_stat_path):
        with open(main_stat_path, "r") as json_file:
            data = json_file.read()
            main_data = json.loads(data)

    if not main_data:
        return jsonify({"error": "Файл не найден"})

    university_stat["main_pps_stat"] = main_data.get("main_pps_stat")
    university_stat["doc_ad_staffing"] = main_data.get("doc_ad_staffing")
    university_stat["avg_gia_mark"] = main_data.get("avg_gia_mark")
    university_stat["mark_stat"] = Dashboard.get_university_mark_stat(lname)
    return jsonify(university_stat)


@dashboard_api.route(
    "/lms/api/v1.0/<lname>/dashboard/faculty_stat/<int:faculty_id>", methods=["GET"]
)
def get_faculty_stat(lname, faculty_id):
    faculty_stat = {}
    main_stat_path = "./static/dashboard/main_stat.json"
    main_data = {}
    if os.path.exists(main_stat_path):
        with open(main_stat_path, "r") as json_file:
            data = json_file.read()
            main_data = json.loads(data)

    if not main_data:
        return jsonify({"error": "Файл не найден"})

    faculty_stat["main_pps_stat"] = main_data.get("main_pps_stat")
    faculty_stat["pps_age_stat"] = main_data.get("pps_age_stat")
    faculty_stat["military_scientific_work"] = main_data.get("military_scientific_work")
    faculty_stat["avg_gia_mark"] = main_data.get("avg_gia_mark")
    faculty_stat["mark_stat"] = Dashboard.get_faculty_mark_stat(
        lname=lname, faculty_id=faculty_id
    )
    return jsonify(faculty_stat)
